Package: rpart.plot
Version: 3.1.4
Title: Plot 'rpart' Models: An Enhanced Version of 'plot.rpart'
Authors@R: person (given = "Stephen",
                   family = "Milborrow",
                   role = c("aut", "cre"),
                   email = "milbo@sonic.net")
Author: Stephen Milborrow [aut, cre]
Maintainer: Stephen Milborrow <milbo@sonic.net>
Depends: R (>= 3.4.0), rpart (>= 4.1-15)
Suggests: earth (>= 5.1.2)
Description: Plot 'rpart' models. Extends plot.rpart() and text.rpart()
             in the 'rpart' package.
License: GPL-3
LazyData: yes
URL: http://www.milbo.org/rpart-plot/index.html
NeedsCompilation: no
Packaged: 2026-01-04 01:37:24 UTC; milbo
Repository: CRAN
Date/Publication: 2026-01-08 08:40:53 UTC
Built: R 4.6.0; ; 2026-04-25 16:12:09 UTC; unix
