\name{show.prp.palettes}
\alias{show.prp.palettes}
\title{Show the built-in prp palettes.}
\description{
Display a diagram showing the built-in palettes
accepted by \code{prp}'s \code{box.palette} argument.
}
\usage{
show.prp.palettes()
}
\examples{
show.prp.palettes()
}
